﻿using System.Net.Sockets;
using NetCore;
using NetCore.ContentParse;
using NetCore.Protocol;

namespace NetServer.NetLink.TCP;

public class TCPServerConnection<T, K> : IContentReceiver, IConnection
    where T : IContentParse where K : IProtocol
{
    public long index;
    public bool isConnected { get; set; }
    private byte[] buffData = new byte[6553];
    private Socket socket;

    //记录半包数据
    private bool isBufferData;
    private byte[] lastBuffData;
    private int lastCount;

    private short lastSendType;

    //数据结束

    //尾包
    private byte[] weiBaoBuffData;


    private Thread _udpClientThread;


    private TCPServer<T, K> _tcpServer;

   
    public IContentParse iContentParse
    {
        get { return _iContentParse; }
    }

    public IProtocol iProtocol
    {
        get { return _iProtocol; }
    }

    private IContentParse _iContentParse;
    public IProtocol _iProtocol;



    public TCPServerConnection(TCPServer<T, K> tcpServer, Socket socket)
    {
        this.socket = socket;
        socket.NoDelay = true;
        _tcpServer = tcpServer;
        isConnected = true;
        _iContentParse = Activator.CreateInstance<T>();
        _iProtocol = Activator.CreateInstance<K>();
        _udpClientThread = new Thread(TheUpdate);
        _udpClientThread.Start();
    }


    private void TheUpdate()
    {
        while (socket != null)
        {
            try
            {
                int count = socket.Receive(buffData);
                if (count <= 0)
                {
                    Thread.Sleep(1);
                    return;
                }

                byte[] data = new byte[count];
                Array.Copy(buffData, 0, data, 0, count);
                _iContentParse.ParseByte(data, _iProtocol, this);
                // LogTool.Log("数据来了" + count);
            }
            catch (Exception e)
            {
                Console.WriteLine(e);
                isConnected = false;
                break;
            }
        }

        Console.WriteLine("服务器执行完毕");
    }

    public void AddContent(object data)
    {
        _tcpServer.LogicalParsing.Logic(data, this);
    }

    public void SendData(object sendData)
    {
        byte[] data = iProtocol.Serialize(sendData);
        byte[] parseData=  _iContentParse.AssembleByte(data);
        socket.SendAsync(parseData, SocketFlags.None);
    }

    public void Update()
    {
    }

    public void ReceiveAsync()
    {
    }

    public int Input(byte[] data)
    {
        return 0;
    }

    public void Dispose()
    {
        socket?.Close();
        socket?.Dispose();
        socket = null;
        _tcpServer = null;
        _udpClientThread = null;
        lastBuffData = null;
        weiBaoBuffData = null;
        buffData = null;
    }
}